<?php
function novashare_network_admin_menu() {

	//add network settings menu item
    add_submenu_page('settings.php', 'Novashare Network Settings', 'Novashare', 'manage_network_options', 'novashare', 'novashare_network_page_callback');

    //create site option if not found
    if(get_site_option('novashare_network') == false) {    
        add_site_option('novashare_network', true);
    }
 
    add_settings_section('novashare_network', 'Network Setup', '__return_false', 'novashare_network');
   
   	//network access
	add_settings_field(
		'access',
		novashare_title(__('Network Access', 'novashare'), 'access', 'https://novashare.io/docs/wordpress-multisite/'),
		'novashare_network_access_callback', 
		'novashare_network', 
		'novashare_network',
		array(
			'tooltip' => __('Choose who has access to manage Novashare plugin settings.', 'novashare')
		)
	);

	//network default
	add_settings_field(
		'default', 
		novashare_title(__('Network Default', 'novashare'), 'default', 'https://novashare.io/docs/wordpress-multisite/'),
		'novashare_network_default_callback', 
		'novashare_network', 
		'novashare_network',
		array(
			'tooltip' => __('Choose a subsite that you want to pull default settings from.', 'novashare')
		)
	);

	//clean uninstall
    add_settings_field(
        'clean_uninstall', 
        novashare_title(__('Clean Uninstall', 'novashare'), 'clean_uninstall', 'https://novashare.io/docs/clean-uninstall/'), 
        'novashare_print_input', 
        'novashare_network', 
        'novashare_network', 
        array(
            'id' => 'clean_uninstall',
            'option' => 'novashare_network',
            'tooltip' => __('When enabled, this will cause all Novashare options data to be removed from your database when the plugin is uninstalled.', 'novashare')
        )
    );

	register_setting('novashare_network', 'novashare_network');
}
add_filter('network_admin_menu', 'novashare_network_admin_menu');
 
//network access
function novashare_network_access_callback($args) {
	$novashare_network = get_site_option('novashare_network');

	echo "<select name='novashare_network[access]' id='access'>";
		echo "<option value=''>" . __('Site Admins (Default)', 'novashare') . "</option>";
		echo "<option value='super' " . ((!empty($novashare_network['access']) && $novashare_network['access'] == 'super') ? "selected" : "") . ">" . __('Super Admins Only', 'novashare') . "</option>";
	echo "<select>";

	//tooltip
    if(!empty($args['tooltip'])) {
        novashare_tooltip($args['tooltip']);
    }
}

//network default
function novashare_network_default_callback($args) {
	$novashare_network = get_site_option('novashare_network');

	echo "<select name='novashare_network[default]' id='default'>";
		$sites = array_map('get_object_vars', get_sites(array('deleted' => 0, 'number' => 1000)));
		if(is_array($sites) && $sites !== array()) {
			echo "<option value=''>" . __('None', 'novashare') . "</option>";
			foreach($sites as $site) {
				echo "<option value='" . $site['blog_id'] . "' " . ((!empty($novashare_network['default']) && $novashare_network['default'] == $site['blog_id']) ? "selected" : "") . ">" . $site['blog_id'] . ": " . $site['domain'] . $site['path'] . "</option>";
			}
		}
	echo "<select>";

	//tooltip
    if(!empty($args['tooltip'])) {
        novashare_tooltip($args['tooltip']);
    }
}
 
//network settings page
function novashare_network_page_callback() {
	if(isset($_POST['novashare_apply_defaults'])) {
		check_admin_referer('novashare-network-apply');
		if(isset($_POST['novashare_network_apply_blog']) && is_numeric($_POST['novashare_network_apply_blog'])) {
			$blog = get_blog_details($_POST['novashare_network_apply_blog']);
			if($blog) {

				//apply default settings to selected blog
				if(is_multisite()) {
					$novashare_network = get_site_option('novashare_network');

					if(!empty($novashare_network['default'])) {

						if($blog->blog_id != $novashare_network['default']) {

							$option_names = array(
								'novashare',
								'novashare_extras'
							);

							foreach($option_names as $option_name) {

								//clear selected blog previous option
								delete_blog_option($blog->blog_id, $option_name);

								//grab new option from default blog
								$new_option = get_blog_option($novashare_network['default'], $option_name);

								//update selected blog with default option
								update_blog_option($blog->blog_id, $option_name, $new_option);

							}

							//Default Settings Updated Notice
							echo "<div class='notice updated is-dismissible'><p>" . __('Default settings applied!', 'novashare') . "</p></div>";
						}
						else {
							//Can't Apply to Network Default
							echo "<div class='notice error is-dismissible'><p>" . __('Select a site that is not already the Network Default.', 'novashare') . "</p></div>";
						}
					}
					else {
						//Network Default Not Set
						echo "<div class='notice error is-dismissible'><p>" . __('Network Default not set.', 'novashare') . "</p></div>";
					}
				}
			}
			else {
				//Blog Not Found Notice
				echo "<div class='notice error is-dismissible'><p>" . __('Error: Blog Not Found.', 'novashare') . "</p></div>";
			}
		}
	}

	//options updated
	if(isset($_GET['updated'])) {
		echo "<div class='notice updated is-dismissible'><p>" . __('Options saved.', 'novashare') . "</p></div>";
	}

	//if no tab is set, default to network tab
	$tab = !empty($_GET['tab']) ? $_GET['tab'] : 'network';

	//network settings wrapper
	echo "<div id='novashare-admin' class='wrap'>";

		//hidden h2 for admin notice placement
		echo "<h2 style='display: none;'></h2>";

		//network tab content
		if($tab == 'network') {

	  		echo "<form method='POST' action='edit.php?action=novashare_update_network_options'>";
			    settings_fields('novashare_network');
			    novashare_settings_section('novashare_network', 'novashare_network');
			    submit_button();
	  		echo "</form>";

	  		echo "<form method='POST' style='clear: both; margin-top: 25px;'>";
		  		echo '<div class="novashare-settings-section">';
		 
		  			echo "<h2>" . __('Apply Default Settings', 'novashare') . "</h2>";
					wp_nonce_field('novashare-network-apply', '_wpnonce', true, true);
					echo "<p>" . __('Select a site from the dropdown and click to apply the settings from your network default (above).', 'novashare') . "</p>";

					echo "<select name='novashare_network_apply_blog' style='margin-right: 10px;'>";
						$sites = array_map('get_object_vars', get_sites(array('deleted' => 0, 'number' => 1000)));
						if(is_array($sites) && $sites !== array()) {
							echo "<option value=''>" . __('Select a Site', 'novashare') . "</option>";
							foreach($sites as $site) {
								echo "<option value='" . $site['blog_id'] . "'>" . $site['blog_id'] . ": " . $site['domain'] . $site['path'] . "</option>";
							}
						}
					echo "<select>";

					echo "<input type='submit' name='novashare_apply_defaults' value='" . __('Apply Default Settings', 'novashare') . "' class='button' />";
				echo '</div>';
			echo "</form>";
		}
		elseif($tab == 'license') {

			//license custom form output
			require_once('license.php');
		}
		elseif($tab == 'support') {

			//support output
			require_once('support.php');
		}

	echo "</div>";
}
 
//update novashare network options
function novashare_update_network_options() {

	//verify post referring page
  	check_admin_referer('novashare_network-options');
 
	//get registered options
	global $new_whitelist_options;
	$options = $new_whitelist_options['novashare_network'];

	//loop through registered options
	foreach($options as $option) {
		if(isset($_POST[$option])) {

			//update site option
			update_site_option($option, $_POST[$option]);
		}
	}

	//redirect to network settings page
	wp_redirect(add_query_arg(array('page' => 'novashare', 'updated' => 'true'), network_admin_url('settings.php')));

	exit;
}
add_action('network_admin_edit_novashare_update_network_options',  'novashare_update_network_options');