<?php
/*
Plugin Name: Novashare
Plugin URI: https://www.persianscript.ir
Description: Novashare is a lightweight and fast social media sharing plugin.
Version: 1.3.7
Author: پرشین اسکریپت
Author URI: https://www.persianscript.ir
License: GPLv2 or later
License URI: http://www.gnu.org/licenses/gpl-2.0.html
Text Domain: novashare
Domain Path: /languages

Novashare uses Font Awesome free SVG icons released under the CC by 4.0 license.
Go to https://github.com/FortAwesome/Font-Awesome for full license details.

The Yummly logo is property of Yummly, Inc. https://www.yummly.com/
*/

//EDD download details
define('NOVASHARE_STORE_URL', 'https://novashare.io/');
define('NOVASHARE_ITEM_ID', 60);
define('NOVASHARE_ITEM_NAME', 'novashare');
define('NOVASHARE_VERSION', '1.3.7');

//load translations
function novashare_load_textdomain() {
	load_plugin_textdomain('novashare', false, dirname(plugin_basename( __FILE__)) . '/languages/');
}
add_action('plugins_loaded', 'novashare_load_textdomain');

//EDD updater function
function novashare_edd_plugin_updater() {

	//to support auto-updates, this needs to run during the wp_version_check cron job for privileged users
	$doing_cron = defined('DOING_CRON') && DOING_CRON;
	if(!current_user_can('manage_options') && !$doing_cron) {
		return;
	}

	//retrieve our license key from the DB
	$novashare_license = is_multisite() ? get_site_option('novashare_license') : get_option('novashare_license');

	$license = !empty($novashare_license['key']) ? trim($novashare_license['key']) : null;
	
	//setup the updater
	$edd_updater = new Novashare_Plugin_Updater(NOVASHARE_STORE_URL, __FILE__, array(
			'version' 	=> NOVASHARE_VERSION,
			'license' 	=> $license,
			'item_id'   => NOVASHARE_ITEM_ID,
			'author' 	=> 'forgemedia',
			'beta'      => false
		)
	);
}
add_action('init', 'novashare_edd_plugin_updater', 0);

//admin settings page
function novashare_admin() {
	include plugin_dir_path(__FILE__) . '/inc/admin.php';
}

//add our admin menus
if(is_admin()) {
	add_action('admin_menu', 'novashare_menu', 9);
}

//admin menu
function novashare_menu() {
	if(novashare_network_access()) {
		$novashare_settings_page = add_options_page('novashare', 'Novashare', 'manage_options', 'novashare', 'novashare_admin');
	}
}

//enqueue admin scripts
function novashare_admin_scripts($hook) {

	if(novashare_network_access()) {

		//settings page + post editor
	    if(in_array($hook, array('settings_page_novashare', 'settings_page_novashare-migrator')) || in_array($hook, array('post.php', 'post-new.php'))) {

	    	//jquery ui
	    	wp_enqueue_script('jquery-ui-sortable');

	    	//color picker ui
	    	wp_enqueue_style('wp-color-picker');

	    	//image upload
	    	wp_enqueue_media();

	    	//admin js
	    	wp_register_script('novashare-admin-js', plugins_url('/js/admin.js', __FILE__), array('wp-color-picker', 'wp-i18n'), NOVASHARE_VERSION, true);
	    	wp_enqueue_script('novashare-admin-js');
	    	wp_set_script_translations('novashare-admin-js', 'novashare', plugin_dir_path(__FILE__) . 'languages');

			//admin css
	    	wp_register_style('novashare-admin-css', plugins_url('/css/admin-style.css', __FILE__), array(), NOVASHARE_VERSION);
			wp_enqueue_style('novashare-admin-css');
	    }

	    //settings page only
	    if($hook === 'settings_page_novashare') {
	    	$cm_settings['codeEditor'] = wp_enqueue_code_editor(array('type' => 'text/css'));
			wp_localize_script('jquery', 'cm_settings', $cm_settings);
			wp_enqueue_script('wp-theme-plugin-editor');
			wp_enqueue_style('wp-codemirror');
	    }
	}
}
add_action('admin_enqueue_scripts', 'novashare_admin_scripts');

//add our plugin non-admin scripts
function novashare_scripts() {

	$novashare = get_option('novashare');
	$allowed = novashare_is_post_allowed();

	//front end css
	if($allowed || !empty($novashare['global_styles'])) {
		wp_register_style('novashare-css', plugins_url('/css/style.min.css', __FILE__), array(), NOVASHARE_VERSION);
		wp_enqueue_style('novashare-css');
		if(!empty($novashare['custom_css'])) {
			wp_add_inline_style('novashare-css', $novashare['custom_css']);
		}        
	}

	//register js
	wp_register_script('novashare-js', plugins_url('/js/novashare.min.js', __FILE__), array(), NOVASHARE_VERSION);

	//inverse hover inline js
	if(!empty($novashare['inline']['inverse_hover']) || !empty($novashare['floating']['inverse_hover'])) {
		wp_add_inline_script('novashare-js', 'document.addEventListener("DOMContentLoaded",function(){document.querySelectorAll(".novashare-inverse-hover .novashare-button").forEach(function(e){e.addEventListener("mouseenter",function(a){if(a.target.classList.contains("novashare-button")){a.target.classList.add("novashare-hover-swap");var n=e.querySelectorAll(".novashare-button-icon.novashare-border");(n.length?n:e.querySelectorAll(".novashare-button-block")).forEach(function(e){e.classList.contains("novashare-inverse")?e.classList.remove("novashare-inverse"):e.classList.add("novashare-inverse")})}},!0),e.addEventListener("mouseout",function(a){if(a.target.classList.contains("novashare-button")){a.target.classList.remove("novashare-hover-swap");var n=e.querySelectorAll(".novashare-button-icon.novashare-border");(n.length?n:e.querySelectorAll(".novashare-button-block")).forEach(function(e){e.classList.contains("novashare-inverse")?e.classList.remove("novashare-inverse"):e.classList.add("novashare-inverse")})}},!0)})});');
	}

	//show pinterest images inline js
	if(!empty($novashare['pinterest']['share_button_behavior']) && $novashare['pinterest']['share_button_behavior'] == 'pinterest') {
		wp_add_inline_script('novashare-js', 'document.addEventListener("DOMContentLoaded",function(){document.querySelectorAll("img:not(.novashare-pinterest-hidden-image)").forEach(function(e){e.setAttribute("data-pin-nopin","1")})});');
	}

	//enqueue js
	if($allowed && !novashare_is_amp()) {
		wp_enqueue_script('novashare-js');
	}

}
add_action('wp_enqueue_scripts', 'novashare_scripts');

//add inline stylesheet for amp reader mode
function novashare_add_amp_css() {

	//make sure shares are enabled for the post
	if(novashare_is_post_allowed()) {
		include 'css/style.min.css';
		$novashare = get_option('novashare');
		if(!empty($novashare['custom_css'])) {
			echo $novashare['custom_css'];
		}       
	}

	//print click to tweet css
	echo novashare_click_to_tweet_css();
}
add_action('amp_post_template_css', 'novashare_add_amp_css');

//add assets to block editor
function novashare_block_editor_assets() {

	$novashare = get_option('novashare');

	//enqueue our block editor scripts + styles
	wp_enqueue_script('novashare-blocks-js', plugins_url('/js/blocks.js', __FILE__), array('wp-blocks', 'wp-element', 'wp-editor', 'wp-i18n'), NOVASHARE_VERSION);
	wp_set_script_translations('novashare-blocks-js', 'novashare', plugin_dir_path(__FILE__) . 'languages');
	wp_enqueue_style('novashare-blocks-css', plugins_url('/css/blocks.css', __FILE__), array( 'wp-edit-blocks' ), NOVASHARE_VERSION);
	wp_add_inline_style('novashare-blocks-css', novashare_click_to_tweet_css());

	//follow block
	wp_enqueue_script('novashare-blocks-follow-js', plugins_url('/js/blocks-follow.js', __FILE__), array('wp-blocks', 'wp-element', 'wp-editor', 'wp-i18n'), NOVASHARE_VERSION);
	wp_localize_script('novashare-blocks-follow-js', 'novashare', novashare_tinymce_localized_settings());

	if(!empty($novashare['pinterest']['image_pins'])) {
		wp_enqueue_script('novashare-blocks-pinterest-js', plugins_url('/js/blocks-pinterest.js', __FILE__), array('wp-blocks', 'wp-element', 'wp-editor', 'wp-i18n'), NOVASHARE_VERSION);
		wp_set_script_translations('novashare-blocks-pinterest-js', 'novashare', plugin_dir_path(__FILE__) . 'languages');
	}

	//pass our plugin options to our block editor script
	wp_localize_script('novashare-blocks-js', 'novashare', novashare_tinymce_localized_settings());
}

//hook assets to editor
add_action('enqueue_block_editor_assets', 'novashare_block_editor_assets');

function mytheme_block_assets() {
	wp_register_style('novashare-css', plugins_url('/css/style.min.css', __FILE__), array(), NOVASHARE_VERSION);
	wp_enqueue_style('novashare-css');
}
add_action( 'enqueue_block_assets', 'mytheme_block_assets' );

//register and render gutenberg blocks
function novashare_register_blocks() {

	$novashare = get_option('novashare');

    register_block_type(
        'novashare/click-to-tweet',
        array(
            'attributes' => array(
                'tweet' => array(
                    'type' => 'string'
                ),
                'cta_text' => array(
                	'type' => 'string'
                ),
                'cta_position' => array(
                	'type' => 'string'
                ),
                'remove_url' => array(
                	'type' => 'boolean',
                	'default' => !empty($novashare['click_to_tweet']['remove_url'])
                ),
                'remove_username' => array(
                	'type' => 'boolean',
                	'default' => !empty($novashare['click_to_tweet']['remove_username'])
                ),
                'hide_hashtags' => array(
                	'type' => 'boolean',
                	'default' => !empty($novashare['click_to_tweet']['hide_hashtags'])
                ),
                'accent_color' => array(
                	'type' => 'string',
                	'default' => !empty($novashare['click_to_tweet']['accent_color']) ? $novashare['click_to_tweet']['accent_color'] : '#00abf0'
                )
            ),
            'render_callback' => 'novashare_click_to_tweet_block',
        )
    );

    register_block_type(
        'novashare/follow',
        array(
            'attributes' => array(
            ),
            'render_callback' => 'novashare_follow_block',
        )
    );

    register_block_type(
        'novashare/follow-network',
        array(
            'attributes' => array(
            ),
            'render_callback' => 'novashare_follow_network_block',
        )
    );
}
add_action('init', 'novashare_register_blocks');

//add buttons to TinyMCE editor
function novashare_tinymce_button() {
	if(current_user_can('edit_posts') && current_user_can('edit_pages')) {
        add_filter('mce_buttons', 'novashare_register_tinymce_button');
        add_filter('mce_external_plugins', 'novashare_add_tinymce_button');
        add_action('before_wp_tiny_mce', 'novashare_before_wp_tiny_mce');
    }
}
add_action('admin_init', 'novashare_tinymce_button');

//register click to tweet button
function novashare_register_tinymce_button($buttons) {
	$buttons[] = 'novashare_click_to_tweet';
	return $buttons;
}

//add click to tweet tinymce js
function novashare_add_tinymce_button($plugin_array) {
	$plugin_array['novashare_click_to_tweet'] = plugins_url('/js/tinymce.js', __FILE__);
	return $plugin_array;
}

//print out inline localized settings for tiny mce (classic editor)
function novashare_before_wp_tiny_mce($mce_settings) {
	$novashare = novashare_tinymce_localized_settings();
	echo "<script>var novashare = " . json_encode($novashare) . ";</script>";
}

//return array of tinymce localized settings + translations
function novashare_tinymce_localized_settings() {

	$novashare = get_option('novashare');

	//translations
	$novashare['translations'] = array(
	    'ctt' => array(
	    	'tooltip' => __('Click to Tweet', 'novashare'),
	    	'title' => __('Click to Tweet Shortcode', 'novashare'),
	    	'submit' => __('Insert Shortcode', 'novashare'),
	    	'body' => array(
	    		'tweet' => __('Tweet', 'novashare'),
	    		'theme' => array(
	    			'title' => __('Theme', 'novashare'),
	    			'values' => array(
	    				'default' => __('Default (Blue Background)', 'novashare'),
	    				'simple' => __('Simple (Transparent Background)', 'novashare'),
	    				'simplealt' => __('Simple Alternate (Gray Background)', 'novashare')
	    			)
	    		),
	    		'ctatext' => __('Call to Action Text', 'novashare'),
	    		'ctaposition' => array(
	    			'title' => __('Call to Action Position', 'novashare'),
	    			'values' => array(
	    				'default' => __('Right (Default)', 'novashare'),
	    				'left' => __('Left', 'novashare')
	    			)
	    		),
	    		'removeurl' => array(
	    			'title' => __('Remove Post URL', 'novashare'),
	    			'text' => __('The URL of the current post will not be added to the tweet.', 'novashare')
	    		),
	    		'removeuser' => array(
	    			'title' => __('Remove Username', 'novashare'),
	    			'text' => __('The Twitter username saved in Novashare will not be added to the tweet.', 'novashare')
	    		),
	    		'hidehash' => array(
	    			'title' => __('Hide Hashtags', 'novashare'),
	    			'text' => __('Trailing hashtags will be hidden from the display box.', 'novashare')
	    		),
	    		'accentcolortext' => __('Accent Color', 'novashare'),
	    		'charcount' => __('Characters Remaining', 'novashare')
	    	)
	    )
	);

	//follow networks
	$novashare['networks']['follow'] = novashare_networks('follow');

	return $novashare;
}

//check multisite and verify access
function novashare_network_access() {
	if(is_multisite()) {
		$novashare_network = get_site_option('novashare_network');
		if((!empty($novashare_network['access']) && $novashare_network['access'] == 'super') && !is_super_admin()) {
			return false;
		}
	}
	return true;
}

//plugins table action links
function novashare_action_links($actions, $plugin_file) {
	if(plugin_basename(__FILE__) == $plugin_file) {

		if(is_network_admin()) {
			$settings_url = network_admin_url('settings.php?page=novashare');
		}
		else {
			$settings_url = admin_url('options-general.php?page=novashare');
		}

		$settings_link = array('settings' => '<a href="' . $settings_url . '">' . __('Settings', 'novashare') . '</a>');
		$actions = array_merge($settings_link, $actions);
	}
	return $actions;
}
add_filter('plugin_action_links', 'novashare_action_links', 10, 5);

//plugins table meta links
function novashare_meta_links($links, $file) {
	if(strpos($file, 'novashare.php' ) !== false) {

		//support link
		$novashare_links = array('<a href="https://novashare.io/docs/" target="_blank">' . __('Support', 'novashare') . '</a>');

		$links = array_merge($links, $novashare_links);
	}
	return $links;
}
add_filter('plugin_row_meta', 'novashare_meta_links', 10, 2);

//header for plugin settings page
function novashare_admin_header() {

	if(empty($_GET['page']) || !in_array($_GET['page'], array('novashare', 'novashare-migrator'))) {
		return;
	}

	$tab = !empty($_GET['tab']) ? $_GET['tab'] : (is_network_admin() ? 'network' : 'options');

	//header container
	echo "<div id='novashare-admin-header'>";

		//logo
		echo '<svg id="novashare-logo" viewBox="0 0 849 605"><g transform="matrix(4.16667,0,0,4.16667,-2334.93,-3853.35)"><path d="M665.014,1007.92C663.741,1008.34 662.477,1008.75 661.202,1009.16C660.454,1009.41 659.695,1009.65 658.945,1009.88C655.969,1007.48 651.874,1006.58 647.976,1007.87C641.852,1009.87 638.509,1016.47 640.517,1022.59C642.524,1028.71 649.118,1032.06 655.242,1030.05C659.19,1028.76 661.982,1025.57 662.928,1021.83C664.935,1021.2 666.938,1020.55 668.941,1019.9C671.479,1019.07 704.677,1008.09 733.536,993.783C735.154,1025.62 715.462,1055.86 683.701,1066.26C658.114,1074.64 631.229,1068.01 612.577,1051.17C608.938,1047.91 605.619,1044.25 602.688,1040.25C578.821,1044.22 563.989,1043.72 560.382,1041.05C561.645,1036.98 572.241,1028.5 591.768,1018.32C608.099,1009.78 630.677,1000.02 659.327,990.641C661.32,989.991 663.333,989.338 665.348,988.705C668.324,991.109 672.42,992.008 676.328,990.724C682.452,988.717 685.795,982.124 683.787,975.999C681.779,969.865 675.186,966.532 669.061,968.53C665.123,969.816 662.34,973.003 661.393,976.742C659.387,977.364 657.394,978.014 655.41,978.664C652.795,979.52 618.352,990.903 589.092,1005.61C585.215,972.076 605.259,939.335 638.556,928.424C665.883,919.474 694.719,927.64 713.412,947.093C716.669,950.452 719.494,953.999 722.067,958.034C745.623,954.164 760.378,954.892 763.94,957.515C762.663,961.651 751.783,970.299 731.696,980.703C715.434,989.13 693.139,998.71 665.014,1007.92Z" style="fill:white;"/></g></svg>';
		echo "<div id='novashare-page-title'>" . ucfirst($tab) . "</div>";

		//callout buttons
		echo "<div id='novashare-admin-header-buttons'>";

			if(is_network_admin()) {
				echo '<a href="?page=novashare&tab=network" class="' . ($tab == 'network' || '' ? 'novashare-active' : '') . '" title="' . __('Network', 'novashare') . '">' . __('Network', 'novashare') . '</a>';
			}
			else {
				echo '<a href="?page=novashare&tab=options" class="' . ($tab == 'options' || '' ? 'novashare-active' : '') . '" title="' . __('Options', 'novashare') . '">' . __('Options', 'novashare') . '</a>';
				echo '<a href="?page=novashare&tab=tools" class="' . ($tab == 'tools' ? 'novashare-active' : '') . '" title="' . __('Tools', 'novashare') . '">' . __('Tools', 'novashare') . '</a>';
			}

			if(!is_plugin_active_for_network('novashare/novashare.php') || is_network_admin()) {
				echo '<a href="?page=novashare&tab=license" class="' . ($tab == 'license' ? 'novashare-active' : '') . '" title="' . __('License', 'novashare') . '">' . __('License', 'novashare') . '</a>';
			}

			echo '<a href="?page=novashare&tab=support" class="' . ($tab == 'support' ? 'novashare-active' : '') . '" title="' . __('Support', 'novashare') . '">' . __('Support', 'novashare') . '</a>';

			echo '<span style="color: rgba(255,255,255,0.5); margin: 0px 10px;" class="novashare-mobile-hide">v' . NOVASHARE_VERSION . '</span>';

			echo "<a href='https://woorkup.com/speed-up-wordpress/?utm_source=novashare&utm_medium=banner&utm_campaign=header-cta' target='_blank' title='" . __('Speed Up Guide', 'novashare') . "' style='background: #fff; color: #4D4595;' class='novashare-mobile-hide'><i class='dashicons dashicons-performance'></i>" . __('Speed Up Guide', 'novashare') . "</a>";
		echo "</div>";
	echo "</div>";
}
add_action('in_admin_header', 'novashare_admin_header', 1);

//install table structure
function novashare_install() {

	global $wpdb;

	$novashare_version = get_option('novashare_version');

	//insert/update custom table
	novashare_install_custom_table($wpdb->prefix);

	//hide hashtags migration
	if($novashare_version < '1.2.3') {
		$novashare = get_option('novashare');
		if(is_array($novashare)) {
			$novashare['click_to_tweet']['hide_hashtags'] = 1;
			update_option('novashare', $novashare);
		}
	}

	//update version
	if($novashare_version != NOVASHARE_VERSION) {
		update_option('novashare_version', NOVASHARE_VERSION, false);
	}

	//update network version if needed
	if(is_multisite()) {
		if(get_site_option('novashare_version') != NOVASHARE_VERSION) {
			update_site_option('novashare_version', NOVASHARE_VERSION, false);
		}
	}
}

//query to insert/update custom table
function novashare_install_custom_table($prefix) {

	global $wpdb;

	$charset_collate = $wpdb->get_charset_collate();

	$sql = "CREATE TABLE {$prefix}novashare_meta (
		id BIGINT(20) NOT NULL AUTO_INCREMENT,
		post_id BIGINT(20),
		meta_key VARCHAR(255) DEFAULT '' NOT NULL,
		meta_value LONGTEXT,
		PRIMARY KEY (id)
	) $charset_collate;";

	require_once(ABSPATH . 'wp-admin/includes/upgrade.php');
	dbDelta($sql);
}

//when new subsite is activated 
function novashare_activate_blog($blog_id) {
	if(!function_exists('is_plugin_active_for_network')) {
		require_once ABSPATH . 'wp-admin/includes/plugin.php';
	}

	if($blog_id instanceof WP_Site) {
		$blog_id = (int) $blog_id->blog_id;
	}

	if(is_plugin_active_for_network('novashare/novashare.php')) {

		global $wpdb;

		switch_to_blog($blog_id);

		//insert/update custom table for blog
		novashare_install_custom_table($wpdb->prefix);

		//update version
		if(get_option('novashare_version') != NOVASHARE_VERSION) {
			update_option('novashare_version', NOVASHARE_VERSION, false);
		}

		restore_current_blog();
	}
}
add_action('wp_initialize_site', 'novashare_activate_blog', 99);
add_action('activate_blog', 'novashare_activate_blog');

//check version for update
function novashare_version_check() {
	$install_flag = false;
	if(is_multisite()) {
		if(get_site_option('novashare_version') != NOVASHARE_VERSION) {
	    	$install_flag = true;
	    }
	}
	if(get_option('novashare_version') != NOVASHARE_VERSION) {
    	$install_flag = true;
    }
	if($install_flag) {
		novashare_install();
	}
}
add_action('plugins_loaded', 'novashare_version_check');

//uninstall plugin + delete options
function novashare_uninstall() {

	global $wpdb;

	//deactivate license if needed
	novashare_deactivate_license();

	//plugin options
	$novashare_options = array(
		'novashare',
		'novashare_tools',
		'novashare_license',
		'novashare_version'
	);

	if(is_multisite()) {
		$novashare_network = get_site_option('novashare_network');
		if(!empty($novashare_network['clean_uninstall']) && $novashare_network['clean_uninstall'] == 1) {
			delete_site_option('novashare_network');

			$sites = array_map('get_object_vars', get_sites(array('deleted' => 0)));
			if(is_array($sites) && $sites !== array()) {
				foreach($sites as $site) {
					foreach($novashare_options as $option) {
						delete_blog_option($site['blog_id'], $option);
					}

					//delete meta table for subsite
					$blog_prefix = $wpdb->get_blog_prefix($site['blog_id']);
	    			$wpdb->query("DROP TABLE IF EXISTS {$blog_prefix}novashare_meta");
				}
			}

			//remove stored version
			delete_site_option('novashare_version');
		}
	}
	else {
		$novashare_tools = get_option('novashare_tools');
		if(!empty($novashare_tools['clean_uninstall']) && $novashare_tools['clean_uninstall'] == 1) {
			foreach($novashare_options as $option) {
				delete_option($option);
			}

			//delete meta table
         	$wpdb->query("DROP TABLE IF EXISTS {$wpdb->prefix}novashare_meta");
		}
	}
}
register_uninstall_hook(__FILE__, 'novashare_uninstall');

//subsite was deleted
function novashare_uninitialize_site($site_id) {
	global $wpdb;

	//delete our custom table for that site
	$wpdb->query("DROP TABLE IF EXISTS {$wpdb->prefix}novashare_meta");
}
add_action('wp_uninitialize_site', 'novashare_uninitialize_site');

//all plugin file includes
include plugin_dir_path(__FILE__) . '/inc/settings.php';
include plugin_dir_path(__FILE__) . '/inc/functions.php';
include plugin_dir_path(__FILE__) . '/inc/functions_pinterest.php';
include plugin_dir_path(__FILE__) . '/inc/click_to_tweet.php';
include plugin_dir_path(__FILE__) . '/inc/share_counts.php';
include plugin_dir_path(__FILE__) . '/inc/share_counts_recovery.php';
include plugin_dir_path(__FILE__) . '/inc/meta.php';
include plugin_dir_path(__FILE__) . '/inc/widget.php';
include plugin_dir_path(__FILE__) . '/inc/network.php';

//load EDD custom updater class
include(dirname(__FILE__) . '/Novashare_Plugin_Updater.php');